﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/04/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Threading.MessagePumping
{
    /// <summary>
    /// Provides a way to consume messages through an event handler.
    /// </summary>
    /// <typeparam name="TMessage">The type of message to process.</typeparam>
    /// <typeparam name="TResult">The type of result to return.</typeparam>
    public sealed class EventConsumableProcessor<TMessage, TResult> : IMessageProcessor<TMessage, TResult>
    {
        /// <summary>
        /// Occurs when a message is executed.
        /// </summary>
        public event EventHandler<MessageExecutingEventArgs<TMessage, TResult>> ExecuteMessage;

        /// <summary>
        /// Executes the specified message.
        /// </summary>
        /// <param name="message">The message to execute.</param>
        /// <returns>The executed message result or null.</returns>
        TResult IMessageProcessor<TMessage, TResult>.Execute(TMessage message)
        {
            if (ExecuteMessage != null)
            {
                //create the event data
                var result = new MessageExecutingEventArgs<TMessage, TResult>(message);

                //execute message
                ExecuteMessage(this, result);

                //return result
                return result.Result;
            }
           
            return default(TResult);
        }
    }
}
