﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/04/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Threading.MessagePumping
{
    /// <summary>
    /// Represents event data for an executing message.
    /// </summary>
    /// <typeparam name="TMessage">The type of messages to execute.</typeparam>
    /// <typeparam name="TResult">The executed message result type.</typeparam>
    public class MessageExecutingEventArgs<TMessage, TResult> : EventArgs
    {
        /// <summary>
        /// Gets the executing message.
        /// </summary>
        public TMessage Message
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the result.
        /// </summary>
        public TResult Result
        {
            get;
            set;
        }

        /// <summary>
        /// Initializes event data with a message.
        /// </summary>
        /// <param name="message">The executing message.</param>
        public MessageExecutingEventArgs(TMessage message)
        {
            Message = message;
        }
    }
}
